importScripts('WAVE.wasm.min.js');

var myInstance;

// WebAssembly
fetch("main.wasm").then(function(response) {
  // binary
  return response.arrayBuffer();
}).then(function(bytes) {
  // compile    
  return WebAssembly.compile(bytes);
}).then(function(module) {
  // instantiate    
  return WebAssembly.instantiate(module);
}).then(function(instance) {
  
  myInstance = instance;  
  
}).catch(function(e){
  console.log(e);
});   
    
onmessage = function (e) {    
      
  // 変換
  if (e.data.state == "run"){
    try{
      var bits = e.data.bits;    
      var tmpL = e.data.L;
      var tmpR = e.data.R;
      var channel = e.data.channel;
      var sampleRate = e.data.sampleRate;      
         
      var data={}, val;
      data.L = new Int32Array(tmpL.length);
      if(tmpR.length == 0){
        data.R = new Int32Array(0);        
      }else{
        data.R = new Int32Array(tmpR.length);        
      }        
      
      // L    
      if(bits == 8){
        for(var i=0; i<tmpL.length;i++){
          if(tmpL[i] >= 0){
            val = tmpL[i] * 127;                      
          }else{
            val = tmpL[i] * 128;
          }        
          data.L[i] = val;
        }
      }else if(bits == 16){
        for(var i=0; i<tmpL.length;i++){
          if(tmpL[i] >= 0){
            val = tmpL[i] * 32767;                      
          }else{
            val = tmpL[i] * 32768;
          }        
          data.L[i] = val;
        }
      }else if(bits == 24){
        for(var i=0; i<tmpL.length;i++){
          if(tmpL[i] >= 0){
            val = tmpL[i] * 8388607;                      
          }else{
            val = tmpL[i] * 8388608;
          }        
          data.L[i] = val;
        }        
      }else if(bits == 32){                
        for(var i=0; i<tmpL.length;i++){
          if(tmpL[i] >= 0){
            val = tmpL[i] * 2147483647;                      
          }else{
            val = tmpL[i] * 2147483648;
          }        
          data.L[i] = val;
        }        
      }

      // R
      if(channel == 2){
        if(bits == 8){
          for(var i=0; i<tmpR.length;i++){
            if(tmpR[i] >= 0){
              val = tmpR[i] * 127;                      
            }else{
              val = tmpR[i] * 128;
            }        
            data.R[i] = val;
          }
        }else if(bits == 16){
          for(var i=0; i<tmpR.length;i++){
            if(tmpR[i] >= 0){
              val = tmpR[i] * 32767;                      
            }else{
              val = tmpR[i] * 32768;
            }        
            data.R[i] = val;
          }
        }else if(bits == 24){
          for(var i=0; i<tmpR.length;i++){
            if(tmpR[i] >= 0){
              val = tmpR[i] * 8388607;                      
            }else{
              val = tmpR[i] * 8388608;
            }        
            data.R[i] = val;
          }        
        }else if(bits == 32){                
          for(var i=0; i<tmpR.length;i++){
            if(tmpR[i] >= 0){
              val = tmpR[i] * 2147483647;                      
            }else{
              val = tmpR[i] * 2147483648;
            }        
            data.R[i] = val;
          }        
        }    
      }           
      
      // フォーマット      
      var WaveFomat ={};
      
      WaveFomat.wFormatTag = 1;    
      WaveFomat.wBitsPerSample = bits;  // 8/16/24/32bit optional     
      WaveFomat.nSamplesPerSec = sampleRate;
      
      WaveFomat.nChannels = channel;
      
      WaveFomat.nBlockAlign = WaveFomat.wBitsPerSample * WaveFomat.nChannels / 8;
      WaveFomat.nAvgBytesPerSec = WaveFomat.nSamplesPerSec * WaveFomat.nBlockAlign; 
      
      var F = new TFileStream();  
            
      // RIFFヘッダ     
      F.WriteString("RIFF");
       
      // ファイルの全体サイズ
      // ※波形データのサイズ + 36byteのヘッダ情報
      F.WriteDWord((data.L.length + data.R.length) * (WaveFomat.wBitsPerSample/ 8) + 36);

      // RIFFの種類(WAVE)
      F.WriteString("WAVE");
     
        // fmtチャンク
        F.WriteString("fmt ");
        
        // チャンクのバイト数
        F.WriteDWord(16);     
                      
        // 種類(1:リニアPCM)
        F.WriteWord(WaveFomat.wFormatTag);
        // チャンネル数(1:モノラル 2:ステレオ)   
        F.WriteWord(WaveFomat.nChannels);
        // サンプリングレート(44100=44.1kHzなど)
        F.WriteDWord(WaveFomat.nSamplesPerSec);
        // 平均データ転送レート(byte/sec) 
        // ※PCMの場合はnSamplesPerSec * nBlockAlign        
        F.WriteDWord(WaveFomat.nAvgBytesPerSec);
        // ブロックサイズ 
        // ※PCMの場合はwBitsPerSample * nChannels / 8         
        F.WriteWord(WaveFomat.nBlockAlign);
        // サンプルあたりのビット数 (bit/sample) 
        // ※PCMの場合は8bit=8, 16bit =16           
        F.WriteWord(WaveFomat.wBitsPerSample);

        // dataチャンク
        F.WriteString("data");
        
        // 波形データのバイト数    
        F.WriteDWord((data.L.length + data.R.length) * (WaveFomat.wBitsPerSample/ 8));

        // --------------
        //  波形データ
        // --------------        
        var step = 4096;        
        
        // バッファ myInstance
        var heap = myInstance.exports.memory.buffer;
                
        var quotient = Math.floor(data.L.length /step); // 商
        var mod = data.L.length % step; // 余り
         
        // アドレス
        var offset_L = myInstance.exports.getAddress_L();
        var offset_R = myInstance.exports.getAddress_R();        
        var offset_dst = myInstance.exports.getAddress_Dst();  
        
        var dst;
        for(var i=0;i<quotient;i++){

          // L          
          var src_L = new Int32Array(heap, offset_L, step);          
          src_L.set(data.L.subarray(i*step,(i*step) + step));
    
          // R    
          if(WaveFomat.nChannels == 2){            
            var src_R = new Int32Array(heap, offset_R, step);          
            src_R.set(data.R.subarray(i*step,(i*step) + step));
          }          
        
          // destination 
          dst = new Uint8Array(heap, offset_dst, step * (bits/8) * WaveFomat.nChannels);    
                
          // WebAssembly run
          myInstance.exports.run(step,bits,WaveFomat.nChannels);    

          F.WriteStream(dst);  
        }
          
        if(mod != 0){
          // L          
          var src_L = new Int32Array(heap, offset_L, mod);          
          src_L.set(data.L.subarray((quotient * step) ,(quotient *step) + mod));
                    
          // R     
          if(WaveFomat.nChannels == 2){                           
            var src_R = new Int32Array(heap, offset_R, mod);          
            src_R.set(data.R.subarray((quotient * step) ,(quotient *step) + mod));
          }
          
          // destination 
          dst = new Uint8Array(heap, offset_dst, mod * (bits/8) * WaveFomat.nChannels);            
          
          // WebAssembly run
          myInstance.exports.run(mod,bits,WaveFomat.nChannels);    

          F.WriteStream(dst);  
        }

      var P = F.Stream.subarray(0, F.getFileSize());   

      postMessage({'state':'ok','stream':P},[P.buffer]);      
    }catch(e){
      postMessage({'state':'no','msg':e});      
    }
  }
};
